import { ChangeDetectionStrategy, ChangeDetectorRef, Component, ElementRef, EventEmitter, Injectable, Input, NgModule, Output, ViewChild, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Subject } from 'rxjs';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class ScriptService {
    constructor() {
        this.loaded = false;
        this.list = {};
        this.emitter = new Subject();
    }
    /**
     * @return {?}
     */
    getChangeEmitter() {
        return this.emitter;
    }
    /**
     * @param {?} path
     * @return {?}
     */
    load(path) {
        if (this.loaded)
            return this;
        this.loaded = true;
        const /** @type {?} */ promises = [];
        path.forEach(script => promises.push(this.loadScript(script)));
        Promise.all(promises).then(res => {
            this.emitter.next(true);
        });
        return this;
    }
    /**
     * @param {?} path
     * @return {?}
     */
    loadScript(path) {
        return new Promise((resolve, reject) => {
            if (this.list[path] === true) {
                resolve(/** @type {?} */ ({
                    path: path,
                    loaded: true,
                    status: 'Loaded',
                }));
                return;
            }
            this.list[path] = true;
            const /** @type {?} */ node = document.createElement('script');
            node.type = 'text/javascript';
            node.src = path;
            node.charset = 'utf-8';
            if (node.readyState) {
                // IE
                node.onreadystatechange = () => {
                    if (node.readyState === 'loaded' || node.readyState === 'complete') {
                        node.onreadystatechange = null;
                        resolve(/** @type {?} */ ({
                            path: path,
                            loaded: true,
                            status: 'Loaded',
                        }));
                    }
                };
            }
            else {
                node.onload = () => {
                    resolve(/** @type {?} */ ({
                        path: path,
                        loaded: true,
                        status: 'Loaded',
                    }));
                };
            }
            node.onerror = (error) => resolve(/** @type {?} */ ({
                path: path,
                loaded: false,
                status: 'Loaded',
            }));
            document.getElementsByTagName('head')[0].appendChild(node);
        });
    }
}
ScriptService.decorators = [
    { type: Injectable },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @record
 */

class UEditorConfig {
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
let _hook_finished = false;
class UEditorComponent {
    /**
     * @param {?} el
     * @param {?} ss
     * @param {?} cog
     * @param {?} cd
     */
    constructor(el, ss, cog, cd) {
        this.el = el;
        this.ss = ss;
        this.cog = cog;
        this.cd = cd;
        this.inited = false;
        this.events = {};
        this.loading = true;
        this.id = `_ueditor-${Math.random()
            .toString(36)
            .substring(2)}`;
        this.loadingTip = '加载中...';
        this.onPreReady = new EventEmitter();
        this.onReady = new EventEmitter();
        this.onDestroy = new EventEmitter();
        this.onContentChange = new EventEmitter();
        this.onChange = Function.prototype;
        this.onTouched = Function.prototype;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.inited = true;
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        // 已经存在对象无须进入懒加载模式
        if (window.UE) {
            this.init();
            return;
        }
        this.ss
            .load(this.cog.js)
            .getChangeEmitter()
            .subscribe(res => {
            this.init();
        });
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if (this.inited && changes["config"]) {
            this.destroy();
            this.init();
        }
    }
    /**
     * @param {?=} options
     * @return {?}
     */
    init(options) {
        if (!window.UE)
            throw new Error('uedito js文件加载失败');
        if (this.instance)
            return;
        // registrer hook
        if (this.cog.hook && !_hook_finished) {
            _hook_finished = true;
            this.cog.hook(UE);
        }
        this.onPreReady.emit(this);
        const /** @type {?} */ opt = Object.assign({}, this.cog.options, this.config, options);
        const /** @type {?} */ ueditor = UE.getEditor(this.id, opt);
        ueditor.ready(() => {
            this.instance = ueditor;
            if (this.value)
                this.instance.setContent(this.value);
            this.onReady.emit(this);
        });
        ueditor.addListener('contentChange', () => {
            this.value = ueditor.getContent();
            this.onChange(this.value);
            this.onTouched(this.value);
            this.onContentChange.emit(this.value);
        });
        this.loading = false;
        this.cd.detectChanges();
    }
    /**
     * @return {?}
     */
    destroy() {
        if (this.instance) {
            for (const /** @type {?} */ ki of this.events) {
                this.instance.removeListener(ki, this.events[ki]);
            }
            this.instance.removeListener('ready');
            this.instance.removeListener('contentChange');
            this.instance.destroy();
            this.instance = null;
        }
        this.onDestroy.emit();
    }
    /**
     * 获取UE实例
     *
     * \@readonly
     * @return {?}
     */
    get Instance() {
        return this.instance;
    }
    /**
     * 设置编辑器语言
     *
     * @param {?} lang
     * @return {?}
     */
    setLanguage(lang) {
        this.ss
            .loadScript(`${this.cog.options.UEDITOR_HOME_URL}/lang/${lang}/${lang}.js`)
            .then(res => {
            this.destroy();
            // 清空语言
            if (!UE._bak_I18N) {
                UE._bak_I18N = UE.I18N;
            }
            UE.I18N = {};
            UE.I18N[lang] = UE._bak_I18N[lang];
            this.init();
        });
    }
    /**
     * 添加编辑器事件
     * @param {?} eventName
     * @param {?} fn
     * @return {?}
     */
    addListener(eventName, fn) {
        if (this.events[eventName])
            return;
        this.events[eventName] = fn;
        this.instance.addListener(eventName, fn);
    }
    /**
     * 移除编辑器事件
     * @param {?} eventName
     * @return {?}
     */
    removeListener(eventName) {
        if (!this.events[eventName])
            return;
        this.instance.removeListener(eventName, this.events[eventName]);
        delete this.events[eventName];
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.destroy();
    }
    /**
     * @return {?}
     */
    _onReuseInit() {
        this.destroy();
        this.init();
    }
    /**
     * @param {?} value
     * @return {?}
     */
    writeValue(value) {
        this.value = value;
        if (this.instance) {
            this.instance.setContent(this.value);
        }
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.onChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
    /**
     * @param {?} isDisabled
     * @return {?}
     */
    setDisabledState(isDisabled) {
        if (isDisabled) {
            this.instance.setDisabled();
        }
        else {
            this.instance.setEnabled();
        }
    }
}
UEditorComponent.decorators = [
    { type: Component, args: [{
                selector: 'ueditor',
                template: `
    <textarea #host id="{{id}}" class="ueditor-textarea"></textarea>
    <div class="loading" *ngIf="loading" [innerHTML]="loadingTip"></div>
    `,
                styles: [`:host .ueditor-textarea{display:none;}`],
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: forwardRef(() => UEditorComponent),
                        multi: true,
                    },
                ],
                changeDetection: ChangeDetectionStrategy.OnPush,
            },] },
];
/** @nocollapse */
UEditorComponent.ctorParameters = () => [
    { type: ElementRef, },
    { type: ScriptService, },
    { type: UEditorConfig, },
    { type: ChangeDetectorRef, },
];
UEditorComponent.propDecorators = {
    "config": [{ type: Input },],
    "loadingTip": [{ type: Input },],
    "host": [{ type: ViewChild, args: ['host',] },],
    "onPreReady": [{ type: Output },],
    "onReady": [{ type: Output },],
    "onDestroy": [{ type: Output },],
    "onContentChange": [{ type: Output },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class UEditorModule {
    /**
     * @param {?} config
     * @return {?}
     */
    static forRoot(config) {
        return {
            ngModule: UEditorModule,
            providers: [{ provide: UEditorConfig, useValue: config }],
        };
    }
}
UEditorModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule],
                providers: [ScriptService],
                declarations: [UEditorComponent],
                exports: [UEditorComponent],
            },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Generated bundle index. Do not edit.
 */

export { UEditorComponent, UEditorConfig, UEditorModule, ScriptService as ɵa };
//# sourceMappingURL=ueditor.js.map
